"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var nodeUtils_1 = require("./nodeUtils");
var defaultOptions = {
    target: ts.ScriptTarget.Latest,
    module: ts.ModuleKind.CommonJS
};
/** Generate documention for all classes in a set of .ts files */
function getDocumentation(fileName, options) {
    if (options === void 0) { options = defaultOptions; }
    var program = ts.createProgram([fileName], options);
    //noinspection TypeScriptUnresolvedFunction
    var checker = program.getTypeChecker();
    var classes = [];
    var interfaces = [];
    //noinspection TypeScriptUnresolvedFunction
    var sourceFile = program.getSourceFile(fileName);
    ts.forEachChild(sourceFile, visit);
    /** visit nodes finding exported classes */
    function visit(node) {
        // Only consider exported nodes
        if (!isNodeExported(node)) {
            return;
        }
        if (node.kind === ts.SyntaxKind.VariableStatement) {
            var classNode = node.declarationList.declarations[0];
            var symbol = classNode.symbol;
            var intf = classNode.initializer.parameters[0].type.typeName.getText();
            var classObj = {
                name: symbol.name,
                comment: ts.displayPartsToString(symbol.getDocumentationComment()),
                extends: 'StatelessComponent',
                propInterface: intf,
            };
            classes.push(classObj);
        }
        if (node.kind === ts.SyntaxKind.ClassDeclaration) {
            var classNode = node;
            var symbol = checker.getSymbolAtLocation(classNode.name);
            var typeArguments = nodeUtils_1.navigate(classNode, ts.SyntaxKind.HeritageClause, ts.SyntaxKind.ExpressionWithTypeArguments);
            var list = nodeUtils_1.getFlatChildren(typeArguments)
                .filter(function (i) { return i.kind === ts.SyntaxKind.Identifier; })
                .map(function (i) { return i.text; });
            classes.push({
                name: symbol.name,
                comment: ts.displayPartsToString(symbol.getDocumentationComment()),
                extends: list.length > 0 && list.indexOf('Component') > -1 ? 'Component' : null,
                propInterface: list.length > 1 ? list[1] : null,
            });
        }
        if (node.kind === ts.SyntaxKind.InterfaceDeclaration) {
            var interfaceDeclaration = node;
            if (interfaceDeclaration.parent === sourceFile) {
                var symbol_1 = checker.getSymbolAtLocation(interfaceDeclaration.name);
                var type = checker.getTypeAtLocation(interfaceDeclaration.name);
                var members = type.getProperties()
                    .filter(function (i) {
                    var s = i;
                    return s.parent && s.parent.name && symbol_1.name === s.parent.name;
                })
                    .map(function (i) {
                    var symbol = checker.getSymbolAtLocation(i.valueDeclaration.name);
                    var prop = i.valueDeclaration;
                    var typeInfo = getType(prop);
                    return {
                        name: i.getName(),
                        text: i.valueDeclaration.getText(),
                        type: typeInfo.type,
                        values: typeInfo.values,
                        isRequired: !prop.questionToken,
                        comment: ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
                    };
                });
                var interfaceDoc = {
                    name: symbol_1.getName(),
                    comment: ts.displayPartsToString(symbol_1.getDocumentationComment()).trim(),
                    members: members,
                };
                interfaces.push(interfaceDoc);
            }
        }
        else if (node.kind === ts.SyntaxKind.ModuleDeclaration) {
            // This is a namespace, visit its children
            ts.forEachChild(node, visit);
        }
    }
    /** True if this is visible outside this file, false otherwise */
    function isNodeExported(node) {
        // Parse the modifier array for the export keyword. If it is found
        // and the node.parent is a sourcefile, return true
        // This only returns top level exports
        var modifiers = node.modifiers;
        if (modifiers) {
            for (var i = 0; i < modifiers.length; i++) {
                if (modifiers[i].kind === ts.SyntaxKind.ExportKeyword) {
                    return node.parent.kind === ts.SyntaxKind.SourceFile;
                }
            }
        }
        return false;
    }
    return {
        classes: classes,
        interfaces: interfaces,
    };
}
exports.getDocumentation = getDocumentation;
function getType(prop) {
    var unionType = prop.type;
    if (unionType && unionType.types) {
        return {
            type: 'string',
            values: unionType.types.map(function (i) { return i.getText(); }),
        };
    }
    //noinspection TypeScriptUnresolvedFunction
    return {
        type: prop.type.getText(),
    };
}
//# sourceMappingURL=parser.js.map